!---------------------------------------------------------------------------------------
! FLEXINVERT: get_initcams
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> get_initcams
!!
!! Purpose:    Reads the initial concentration files and interpolates these to
!!             the flexpart grid
!!
!! Interface:
!!
!!    Inputs
!!             filename  -  name of 4D concentration file
!!             varname   -  variable name
!!             concini   -  concentrations on flexpart grid (zero array on input)
!! 
!!    Externals
!!             convertgrid
!!             interpz
!!             check
!!             
!---------------------------------------------------------------------------------------

subroutine get_initcams(filename, varname, ntime, concini)

  use mod_var
  use mod_tools,   only : sort
  use mod_save,    only : check
  use netcdf
  
  implicit none

  character(len=max_path_len),                 intent(in)     :: filename
  character(len=max_name_len),                 intent(in)     :: varname
  integer,                                     intent(in)     :: ntime
  real, dimension(nxgrid,nygrid,nzgrid,ntime), intent(in out) :: concini

  character(len=*), parameter                                 :: lonname='longitude'
  character(len=*), parameter                                 :: latname='latitude'
  character(len=*), parameter                                 :: timename='time'
  character(len=*), parameter                                 :: prsname='hlevel' ! hlevel for CH4, level for N2O
  logical, parameter                                          :: midpoints=.true. ! true for CH4, false for N2O
  integer                                                     :: ncid, xid, yid, zid, tid, apid, bpid, varid
  integer                                                     :: n, kz, status
  integer                                                     :: nx, ny, nz, nt, nlev, nzmax 
  integer                                                     :: nx_out, ny_out
  logical                                                     :: flag_hyb
  real, dimension(:), allocatable                             :: lon, lat, lonin, latin
  real(kind=8), dimension(:), allocatable                     :: lono, lato
  real, dimension(:), allocatable                             :: ap, bp, pres, mpres, alt
  integer, dimension(:), allocatable                          :: indx, indy
  real, dimension(nxgrid)                                     :: lon_out
  real, dimension(nygrid)                                     :: lat_out
  real, dimension(:,:,:), allocatable                         :: work1, work2, work3, work4
  real, dimension(:,:,:,:), allocatable                       :: conc, concin
  real                                                        :: sclfact, offset

  ! initialize
  ! ----------

  ! don't pass globals to subroutines
  lon_out = gbl_lon
  lat_out = gbl_lat
  nx_out = nxgrid
  ny_out = nygrid

  ! read netcdf file
  ! ----------------

  write(logid,*) 'Reading file: '//trim(filename)

  ! read dimensions
  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,lonname,xid) )
  call check( nf90_inq_dimid(ncid,latname,yid) )
  call check( nf90_inq_dimid(ncid,prsname,zid) )
  call check( nf90_inq_dimid(ncid,timename,tid) )
  call check( nf90_inquire_dimension(ncid,xid,len=nx) )
  call check( nf90_inquire_dimension(ncid,yid,len=ny) )
  call check( nf90_inquire_dimension(ncid,zid,len=nlev) )
  call check( nf90_inquire_dimension(ncid,tid,len=nt) )
 
  if ( ntime.ne.nt ) write(logid,*) 'ERROR get_initcams: inconsistent time dimension'

  allocate( lon(nx) )
  allocate( lat(ny) )

  ! read variables
  call check( nf90_inq_varid(ncid,lonname,xid) )
  call check( nf90_get_var(ncid,xid,lon) )
  call check( nf90_inq_varid(ncid,latname,yid) )
  call check( nf90_get_var(ncid,yid,lat) )
  ! check if contains hybrid pressure coordinates 
  status = nf90_inq_varid(ncid,'hyai',apid)
  print*, 'get_initcams: status = ',status
  if (status.eq.-49) then
    ! no hybrid pressure coordinates
    flag_hyb = .false.
    nz = nlev
    allocate( mpres(nz) )
    call check( nf90_inq_varid(ncid,prsname,zid) )
    call check( nf90_get_var(ncid,zid,mpres) )
  else
    ! contains hybrid pressure coordinates
    flag_hyb = .true.
    nz = nlev - 1
    allocate( ap(nlev) )
    allocate( bp(nlev) )
    allocate( pres(nlev) )
    allocate( mpres(nz) )
    call check( nf90_inq_varid(ncid,'hyai',apid) )
    call check( nf90_get_var(ncid,apid,ap) )
    call check( nf90_inq_varid(ncid,'hybi',bpid) )
    call check( nf90_get_var(ncid,bpid,bp) )
  endif
  allocate( alt(nz) )
  allocate( conc(nx,ny,nz,nt) )
  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,conc(:,:,:,:)) )
  ! check if file contains scalar and offset
  status = nf90_inquire_attribute(ncid,varid,'scale_factor')
  if ( status.eq.nf90_noerr ) then
    status = nf90_inquire_attribute(ncid,varid,'add_offset')
    if ( status.eq.nf90_noerr ) then
      call check( nf90_get_att(ncid, varid, 'scale_factor', sclfact) )
      call check( nf90_get_att(ncid, varid, 'add_offset', offset) )
      conc = conc*sclfact + offset
    endif
  endif
  call check( nf90_close(ncid) )

  ! adjust lat and lon of grid
  allocate( indy(ny) )
  allocate( indx(nx) )
  allocate( lato(ny) )
  allocate( lono(nx) )
  lato = dble(lat)
  lono = dble(lon)
  ! sort from south to north and west to east
  call sort(ny, lato, indy)
  call sort(nx, lono, indx)
  conc = conc(indx,indy,:,:)
  print*, 'get_initcams: indy = ',indy
  print*, 'get_initcams: indx = ',indx
  ! cut extra longitude grid cells (assumes grid is -180 to 180 and not 0 to 360)
  if ( lono(nx).ge.180. ) then 
    allocate( concin((nx-1),ny,nz,nt) )
    allocate( lonin((nx-1)) )
    concin = conc(1:(nx-1),:,:,:)
    lonin = sngl(lono(1:(nx-1)))
  else  
    allocate( concin(nx,ny,nz,nt) )
    allocate( lonin(nx) )
    concin = conc(:,:,:,:)
    lonin = sngl(lono(:))
  endif
  allocate( latin(ny) )
  if ( lato(ny).eq.90. ) then
    do n = 1, ny
      latin(n) = -90. + real(n - 1)*(180./real(ny))
    end do
  else
    latin = sngl(lato)
  endif  
  print*, 'get_initcams: latin = ',latin
  print*, 'get_initcams: lonin = ',lonin

  ! calculate mid layer pressure
  ! (for simplicity currently use standard surface pressure
  ! should be good enough for background calculation,
  ! future update would be to read surf pres from file and do 
  ! vertical interpolation to flexpart levels grid by grid)
  if ( flag_hyb ) then
    do n = 1, nlev
      pres(n) = ap(n) + bp(n)*psurf
    end do
    ! avoid pressure of zero
    pres(nlev) = 2.
    do n = 1, nz
      mpres(n) = 0.5*(pres(n) + pres(n+1))
    end do 
  endif

  do n = 1, nz
    alt(n)=7500*log(psurf/mpres(n))
  end do
  if ( flag_hyb ) print*, 'get_initcams: pres = ',pres
  print*, 'get_initcams: mpres = ',mpres
  print*, 'get_initcams: alt = ',alt

  ! interpolate to flexpart grid
  ! ----------------------------

  if ( lono(nx).ge.180. ) then  
    allocate( work1((nx-1),ny,1) )
    nx = nx - 1
  else
    allocate( work1(nx,ny,1) )
  endif

  ! determine upper alt level of conc field for vertical interpolation
  nzmax = minloc(alt, dim=1, mask=abs(alt-outheight(nzgrid)).eq.minval(abs(alt-outheight(nzgrid))))
  nzmax = min(nz, nzmax+1)
  print*, 'get_initcams: nzmax = ',nzmax

  allocate( work2(nxgrid,nygrid,1) )
  allocate( work3(nxgrid,nygrid,nzmax) )
  allocate( work4(nxgrid,nygrid,nzgrid) )

  do n = 1, nt
    ! interpolate horizontally
    do kz = 1, nzmax
      work1(:,:,1) = concin(:,:,kz,n)
      ! gbl_lon and gbl_lat are lon and lat of global flexpart domain
      call convertgrid(midpoints, nx, ny, lonin, latin, work1, nx_out, ny_out, lon_out, lat_out, work2)
      work3(:,:,kz) = work2(:,:,1)
    end do
    ! interpolate vertically
    call interpz(nzmax, alt(1:nzmax), work3, work4)
    concini(:,:,:,n) = work4
  end do

  deallocate( lon )
  deallocate( lat )
  deallocate( lono )
  deallocate( lato )
  deallocate( lonin )
  deallocate( latin )
  deallocate( indx )
  deallocate( indy )
  if ( allocated(ap) )   deallocate( ap )
  if ( allocated(bp) )   deallocate( bp )
  if ( allocated(pres) ) deallocate( pres )
  deallocate( mpres )
  deallocate( alt )
  deallocate( conc )
  deallocate( concin )
  deallocate( work1 )
  deallocate( work2 )
  deallocate( work3 )
  deallocate( work4 )

end subroutine get_initcams


