!---------------------------------------------------------------------------------------
! FLEXINVERT: get_initfpctm_hour
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> get_initfpctm_hour
!!
!! Purpose:    Reads the initial concentration files and interpolates these to
!!             the flexpart grid
!!
!! Interface:
!!
!!    Inputs
!!             filename  -  name of 4D concentration file
!!             varname   -  variable name
!!             concini   -  concentrations on flexpart grid (zero array on input)
!! 
!!    Externals
!!             convertgrid
!!             interpz
!!             check
!!             
!---------------------------------------------------------------------------------------

subroutine get_initfpctm_hour(filename, varname, concini)

  use mod_var
  use mod_save,    only : check
  use netcdf
  
  implicit none

  character(len=max_path_len),                  intent(in)     :: filename
  character(len=max_name_len),                  intent(in)     :: varname
  real, dimension(nxgrid,nygrid,nzgrid,nztime), intent(in out) :: concini

  character(len=*), parameter                           :: lonname='longitude'
  character(len=*), parameter                           :: latname='latitude'
  character(len=*), parameter                           :: altname='lev'
  character(len=*), parameter                           :: timename='Date'
  integer                                               :: ncid, xid, yid, zid, tid, varid
  integer                                               :: kz, it
  integer                                               :: nx, ny, nz, nt 
  integer                                               :: nx_out, ny_out
  logical                                               :: midpoints
  real, dimension(:), allocatable                       :: lon, lat, alt, time
  real, dimension(nxgrid)                               :: lon_out
  real, dimension(nygrid)                               :: lat_out
  real, dimension(:,:,:), allocatable                   :: work, workout, workout2, concout
  real, dimension(:,:,:,:), allocatable                 :: conc

  ! initialize
  ! ----------

  ! don't pass globals to subroutines
  lon_out = gbl_lon
  lat_out = gbl_lat
  nx_out = nxgrid
  ny_out = nygrid

  ! read netcdf file
  ! ----------------

  write(logid,*) 'Reading file: '//trim(filename)

  ! read dimensions
  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,lonname,xid) )
  call check( nf90_inq_dimid(ncid,latname,yid) )
  call check( nf90_inq_dimid(ncid,altname,zid) )
  call check( nf90_inq_dimid(ncid,timename,tid) )
  call check( nf90_inquire_dimension(ncid,xid,len=nx) )
  call check( nf90_inquire_dimension(ncid,yid,len=ny) )
  call check( nf90_inquire_dimension(ncid,zid,len=nz) )
  call check( nf90_inquire_dimension(ncid,tid,len=nt) )

  allocate( lon(nx) )
  allocate( lat(ny) )
  allocate( alt(nz) )
  allocate( time(nt) )
  allocate( conc(nx,ny,nz,nt) )
  allocate( work(nx,ny,1) )
  allocate( workout(nxgrid,nygrid,1) ) 
  allocate( workout2(nxgrid,nygrid,nz) ) 
  allocate( concout(nxgrid,nygrid,nzgrid) )

  ! read variables
  call check( nf90_inq_varid(ncid,lonname,xid) )
  call check( nf90_get_var(ncid,xid,lon) )
  call check( nf90_inq_varid(ncid,latname,yid) )
  call check( nf90_get_var(ncid,yid,lat) )
  call check( nf90_inq_varid(ncid,altname,zid) )
  call check( nf90_get_var(ncid,zid,alt) )
  call check( nf90_inq_varid(ncid,timename,tid) )
  call check( nf90_get_var(ncid,tid,time) )
  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,conc(:,:,:,:)) )
  call check( nf90_close(ncid) )

  ! interpolate to flexpart grid
  ! ----------------------------
  
  midpoints = .true.
  do it = 1, nt
    ! interpolate horizontally
    do kz = 1, nz
      work(:,:,1) = conc(:,:,kz,it)
      ! lon_out and lat_out are lon and lat of global flexpart domain
      call convertgrid(midpoints, nx, ny, lon, lat, work, nx_out, ny_out, lon_out, lat_out, workout)
      workout2(:,:,kz) = workout(:,:,1)
    end do
    ! interpolate vertically
    call interpz(nz, alt, workout2, concout)
    concini(:,:,:,it) = concout
  end do

  deallocate( lon )
  deallocate( lat )
  deallocate( alt )
  deallocate( conc )
  deallocate( work )
  deallocate( workout )
  deallocate( workout2 )
  deallocate( concout )

end subroutine get_initfpctm_hour


