!---------------------------------------------------------------------------------------
! FLEXINVERT: init_co2
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> init_co2
!! Initialize CO2 variables
!! 
!! Purpose:    Reads prior fluxes and observations of CO2 and calculates the 
!!             state and observation vectors as well as their uncertainties.
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             config  -  configuration settings data structure
!!             fluxes  -  flux data structure
!!             obs     -  observation data structure
!!             covar   -  error covariance data structure
!!
!!    Externals
!!             alloc_fluxes
!!             alloc_states
!!             caldate     
!!             read_ncdf        
!!             read_bg       
!!             correl
!!             error_cov        
!!             read_obs
!!             assign_obs
!!             init_cini
!!             perturb_state
!!             perturb_obs
!!
!---------------------------------------------------------------------------------------

subroutine init_co2(files, config, fluxes, obs, states, covar)

  use mod_var
  use mod_settings
  use mod_strings
  use mod_dates
  use mod_tools
  use mod_fluxes
  use mod_obs
  use mod_states
  use mod_covar
  use mod_perturb
  use mod_save, only : save_for_restart

  implicit none

  type (files_t),         intent (in)                    :: files
  type (config_t),        intent (in)                    :: config
  type (fluxes_t),        intent (in out)                :: fluxes
  type (obs_t),           intent (in out)                :: obs
  type (states_t),        intent (in out)                :: states
  type (covar_t),         intent (in out)                :: covar

  character(max_path_len)                                :: filename
  character(max_path_len)                                :: rowfmt
  character(len=4)                                       :: adate
  character(len=2)                                       :: amonth
  logical                                                :: lexist
  real, dimension(nbox,ntstate,ndt)                      :: err_box
  real, dimension(nbox,ntstate,nd_nee)                   :: nee_box
  real, dimension(nbox,nbox)                             :: corr
  real(kind=8), dimension(ntstate)                       :: statime
  real(kind=8), dimension(:), allocatable                :: timeff, timefp
  real, dimension(nxregrid,nyregrid,nd_nee)              :: reg_nee
  real(kind=8), dimension(int(statres)*nd_nee)           :: timenee
  real, dimension(nxregrid,nyregrid,int(statres)*nd_nee) :: reg_flx
  real, dimension(:,:,:), allocatable                    :: flx
  real, dimension(:), allocatable                        :: xerr
  integer, dimension(ndt)                                :: cnt
  integer                                                :: nread, num
  integer                                                :: eomday
  integer                                                :: jjjjmmdd, hhmiss, hh
  real                                                   :: area
  real(kind=8)                                           :: jd, jdi 
  integer                                                :: ix, jy, ix0, jy0
  integer                                                :: n, nb, nd, nt, i
  integer                                                :: ierr
  real(kind=8), dimension(:), allocatable                :: jdate
  character(len=3), dimension(:), allocatable            :: recs
  real                                                   :: conc
  character(len=20)                                      :: dump
  real, dimension(:,:), allocatable                      :: datain, dataout
  real, dimension(nxregrid,nyregrid)                     :: tmp

  ! global prior fluxes
  ! -------------------

  ! global fluxes are used for calculating contribution to mixing ratios
  ! from outside the inversion domain i.e. background

  call alloc_fluxes(config, fluxes)
  fluxes%flxnee(:,:,:) = 0.
  fluxes%flxff(:,:,:) = 0.
  fluxes%flxocn(:,:,:) = 0.
  fluxes%time(:) = 0.

  ! loop over global prior flux files

  n = 0
  jd = juldatei

!  do while ( (jd.le.juldatef).and.(n.lt.ntflx) )
  do while ( (jd.le.juldatef).and.(n.lt.ntstate) )

    n = n + 1
    call caldate(jd, jjjjmmdd, hhmiss)
    write(adate,'(I4.4)') jjjjmmdd/10000
    write(amonth,'(I2.2)') (jjjjmmdd-(jjjjmmdd/10000)*10000)/100
!    eomday = calceomday(jjjjmmdd/100)

    !--- NEE fluxes

    ! number of fields to read for each flux time step
!    nread = eomday*nd_nee
    nread = int(statres)*nd_nee
    print*, 'init_co2: n = ',n
    print*, 'init_co2: nee nread = ',nread
    allocate( flx(nxgrid,nygrid,nread), stat = ierr )
    if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
    filename = str_replace(files%filename_nee, 'YYYY', adate)
    filename = trim(files%path_prior)//trim(filename)
    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then    
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    write(logid,*) 'Reading NEE :'//trim(filename)
    call read_ncdf(filename, &
                   files%varname_nee, &
                   files%lonname_nee, &
                   files%latname_nee, &
                   files%timename_nee,&
                   llx, lly, &
                   nxgrid, nygrid, &
                   jd, nread, num, &
                   flx)
    print*, 'init_co2: num = ',num
    ! convert to kg/m2/s
    flx = flx/3600.
    ! apply numerical scaling
    flx = flx*numscale
    ! average field to flux time step 
    fluxes%flxnee(:,:,n) = sum(flx,dim=3)/real(nread)
    deallocate( flx )

    !--- fossil fuel fluxes

    ! number of fields to read for each flux time step
 !   nread = eomday*24
   nread = int(statres)*24 
   print*, 'init_co2: ff nread = ',nread
    allocate( flx(nxgrid,nygrid,nread), stat = ierr )
    if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
    filename = str_replace(files%filename_ff, 'YYYY', adate)
    filename = str_replace(filename, 'MM', amonth)
    filename = trim(files%path_prior)//trim(filename)
    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then    
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    write(logid,*) 'Reading fossil fuel :'//trim(filename)
    ! timestamp is hours since 1-Jan each year
    jdi = (jd - juldate((jjjjmmdd/10000)*10000+101,0))*24d0 + juldate(19000101,0)
    call read_ncdf(filename, &
                   files%varname_ff, &
                   files%lonname_ff, &
                   files%latname_ff, &
                   files%timename_ff,&
                   llx, lly, &
                   nxgrid, nygrid,  &
                   jdi, nread, num, &
                   flx)
    print*, 'init_co2: num = ',num
    ! convert from micromol/m2/s to kg/m2/s
    flx = flx*12./1.e9
    ! apply numerical scaling
    flx = flx*numscale
    ! average field to flux time step 
    fluxes%flxff(:,:,n) = sum(flx,dim=3)/real(nread)
    deallocate( flx )

    !--- ocean fluxes

    ! number of fields to read for each flux time step
!    nread = eomday/int(365/config%nt_flx)
    nread = int(statres)/int(365/config%nt_flx)
    if ( nread.eq.0 ) nread = 1
    allocate( flx(nxgrid,nygrid,nread), stat = ierr )
    if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
    filename = str_replace(files%filename_ocn, 'YYYY', adate)
    filename = trim(files%path_prior)//trim(filename)
    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then    
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    write(logid,*) 'Reading ocean :'//trim(filename)
    call read_ncdf(filename, &
                   files%varname_ocn, &
                   files%lonname_ocn, &
                   files%latname_ocn, &
                   files%timename_ocn,&
                   llx, lly, &
                   nxgrid, nygrid, &
                   jd, nread, num, &
                   flx)
    ! convert to kg/m2/s
    flx = flx/3600.
    ! apply numerical scaling
    flx = flx*numscale
    ! average field to flux time step 
    fluxes%flxocn(:,:,n) = sum(flx,dim=3)/real(nread)
    deallocate( flx )

    ! flux time stamp is start of time step
    fluxes%time(n) = jd

!    jd = jd + dble(eomday)  
    jd = jd + dble(statres)

  end do 

  ! regional prior fluxes
  ! ---------------------

  !--- fossil fuel

  fluxes%flxff_nest(:,:,:) = 0.
  n = 0
  jd = juldatei
  do while ( jd.le.juldatef )
    n = n + 1
    call caldate(jd, jjjjmmdd, hhmiss)
    write(adate,'(I4.4)') jjjjmmdd/10000
    write(amonth,'(I2.2)') (jjjjmmdd-(jjjjmmdd/10000)*10000)/100
!    eomday = calceomday(jjjjmmdd/100)
    ! number of fields to read for each flux time step
!    nread = eomday*24
    nread = int(statres)*24
    print*, 'init_co2: ffreg nread = ',nread
    allocate( flx(nxregrid,nyregrid,nread), stat = ierr )
    allocate( timeff(nread) )
    if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
    filename = str_replace(files%filenest_ff, 'YYYY', adate)
    filename = str_replace(filename, 'MM', amonth)
    filename = trim(files%path_prior)//trim(filename)
    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    write(logid,*) 'Reading fossil fuel :'//trim(filename)
    ! timestamp is hours since 1-Jan each year
    jdi = (jd - juldate((jjjjmmdd/10000)*10000+101,0))*24d0 + juldate(19000101,0)
    call read_ncdf(filename, &
                   files%varnest_ff, &
                   files%lonnest_ff, &
                   files%latnest_ff, &
                   files%timenest_ff,&
                   rllx, rlly, &
                   nxregrid, nyregrid, &
                   jdi, nread, num, &
                   flx)
    print*, 'init_co2: num = ',num
    ! convert from micromol/m2/s to kg/m2/s
    flx = flx*12./1.e9
    ! apply numerical scaling
    flx = flx*numscale
    ! fossil fuel timestamp julian days
!    do i = 1, 24*eomday
    do i = 1, 24*int(statres)
      timeff(i) = jd + dble(i-1)/24d0
    end do
    print*, 'init_co2: timeff = ',timeff
    ! average/interpolate to footprint timestamp
    if ( ndgrid.eq.24 ) then
      ! no averaging/interpolation needed
!      fluxes%flxff_nest(:,:,(n-1)*ndgrid*eomday+1:n*ndgrid*eomday) = flx
!      fluxes%timefp((n-1)*eomday*ndgrid+1:n*eomday*ndgrid) = timeff
      fluxes%flxff_nest(:,:,(n-1)*ndgrid*int(statres)+1:n*ndgrid*int(statres)) = flx
      fluxes%timefp((n-1)*int(statres)*ndgrid+1:n*int(statres)*ndgrid) = timeff
    else
      ! do averaging/interpolation
!      allocate( datain(nxregrid,eomday*24) )
!      allocate( dataout(nxregrid,eomday*ndgrid) )
!      allocate( timefp(eomday*ndgrid) )
      allocate( datain(nxregrid,int(statres)*24) )
      allocate( dataout(nxregrid,int(statres)*ndgrid) )
      allocate( timefp(int(statres)*ndgrid) )
!      do i = 1, eomday*ndgrid
      do i = 1, int(statres)*ndgrid
        timefp(i) = jd + dble(i-1)/dble(ndgrid)
      enddo
      if ( ndgrid.lt.24 ) then
        ! average flux
        do jy = 1, nyregrid
          datain = flx(:,jy,:)
!          call average(nxregrid, eomday*24, timeff, datain, eomday*ndgrid, timefp, dataout)
!          fluxes%flxff_nest(:,jy,(n-1)*ndgrid*eomday+1:n*ndgrid*eomday) = dataout(:,:)
          call average(nxregrid, int(statres)*24, timeff, datain, int(statres)*ndgrid, timefp, dataout)
          fluxes%flxff_nest(:,jy,(n-1)*ndgrid*int(statres)+1:n*ndgrid*int(statres)) = dataout(:,:)
        end do
      else if ( ndgrid.gt.24 ) then
        ! interpolate flux
        do jy = 1, nyregrid
          datain = flx(:,jy,:)
!          call interp(nxregrid, eomday*24, timeff, datain, eomday*ndgrid, timefp, dataout)
!          fluxes%flxff_nest(:,jy,(n-1)*ndgrid*eomday+1:n*ndgrid*eomday) = dataout(:,:)
          call interp(nxregrid, int(statres)*24, timeff, datain, int(statres)*ndgrid, timefp, dataout)
          fluxes%flxff_nest(:,jy,(n-1)*ndgrid*int(statres)+1:n*ndgrid*int(statres)) = dataout(:,:)
        end do
      endif
!      fluxes%timefp((n-1)*eomday*ndgrid+1:n*eomday*ndgrid) = timefp
      fluxes%timefp((n-1)*int(statres)*ndgrid+1:n*int(statres)*ndgrid) = timefp
      deallocate( datain )
      deallocate( dataout )
      deallocate( timefp )
    endif
    deallocate( flx )
    deallocate( timeff )
!    jd = jd + dble(eomday)
    jd = jd + dble(statres)
  end do

  !--- ocean fluxes

  if ( config%nested ) then
    ! nested
    fluxes%flxocn_nest(:,:,:) = 0.
    n = 0
    jd = juldatei
!    do while ( (jd.le.juldatef).and.(n.lt.ntflx) )
    do while ( (jd.le.juldatef).and.(n.lt.ntstate) )
      n = n + 1
      call caldate(jd, jjjjmmdd, hhmiss)
      write(adate,'(I4.4)') jjjjmmdd/10000
!      eomday = calceomday(jjjjmmdd/100)
      ! number of fields to read for each flux time step
!      nread = eomday/int(365/config%nt_flx)
      nread = int(statres)/int(365/config%nt_flx)
      if ( nread.eq.0 ) nread = 1
      allocate( flx(nxregrid,nyregrid,nread), stat = ierr )
      if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
      filename = str_replace(files%filenest_ocn, 'YYYY', adate)
      filename = trim(files%path_prior)//trim(filename)
      inquire(file=trim(filename),exist=lexist)
      if (.not.lexist) then
        write(logid,*) 'ERROR: cannot find '//trim(filename)
        stop
      endif
      write(logid,*) 'Reading regional ocean :'//trim(filename)
      call read_ncdf(filename, &
                     files%varnest_ocn, &
                     files%lonnest_ocn, &
                     files%latnest_ocn, &
                     files%timenest_ocn,&
                     rllx, rlly, &
                     nxregrid, nyregrid, &
                     jd, nread, num, &
                     flx)
      ! convert to kg/m2/s
      flx = flx/3600.
      ! apply numerical scaling
      flx = flx*numscale
      ! average field to flux time step 
      fluxes%flxocn_nest(:,:,n) = sum(flx,dim=3)/real(nread)
      deallocate( flx )
!      jd = jd + dble(eomday)
      jd = jd + dble(statres)
    end do
  endif

  !--- NEE fluxes 

  ! read hourly NEE and calculate uncertainty for boxes

  write(logid,*) 'Reading hourly NEE'
  fluxes%flxnee_nest(:,:,:) = 0.
  err_box(:,:,:) = 0.
  nee_box(:,:,:) = 0.

  jd = juldatei
  nd = 1

  do while ( (jd.le.(juldatef-dble(statres)+1d0)).and.(nd.le.ntstate) )

    print*, 'init_co2: hourly nee nd = ',nd
    reg_flx(:,:,:) = 0.
    call caldate(jd, jjjjmmdd, hhmiss)
    write(adate,'(I4.4)') jjjjmmdd/10000

    ! read NEE fluxes for current day (starting from 00:00)
    filename = str_replace(files%filenest_nee, 'YYYY', adate)
    filename = trim(files%path_prior)//trim(filename)
    print*, filename
    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    call read_ncdf(filename, &
                   files%varnest_nee, &
                   files%lonnest_nee, &
                   files%latnest_nee, &
                   files%timenest_nee,&
                   rllx, rlly, &
                   nxregrid, nyregrid, &
                   jd, int(statres)*nd_nee, num, &
                   reg_flx)
    ! check if everything read
    print*, 'init_co2: num = ',num
    if ( num.ne.int(statres)*nd_nee ) then
      jdi = jd + num/nd_nee
      nread = int(statres)*nd_nee - num
      allocate( flx(nxregrid,nyregrid,nread), stat = ierr )
      if ( ierr.ne.0 ) stop 'ERROR init_co2: not enough memory'
      ! read remaining days from next file
      call caldate(jdi, jjjjmmdd, hhmiss)
      write(adate,'(I4.4)') jjjjmmdd/10000
      filename = str_replace(files%filenest_nee, 'YYYY', adate)
      filename = trim(files%path_prior)//trim(filename)
      inquire(file=trim(filename),exist=lexist)
      if (.not.lexist) then
        write(logid,*) 'ERROR: cannot find '//trim(filename)
        stop
      endif
      call read_ncdf(filename, &
                     files%varnest_nee, &
                     files%lonnest_nee, &
                     files%latnest_nee, &
                     files%timenest_nee,&
                     rllx, rlly, &
                     nxregrid, nyregrid, &
                     jdi, nread, num, &
                     flx)
      reg_flx(:,:,int(statres)*nd_nee-nread+1:int(statres)*nd_nee) = flx(:,:,:)
      deallocate( flx )
    endif
    ! convert to kg/m2/s
    reg_flx = reg_flx/3600.
    ! apply numerical scaling
    reg_flx = reg_flx*numscale
    
    do i = 1, int(statres)*nd_nee
      timenee(i) = jd + dble(i-1)/dble(nd_nee)
    end do
    print*, 'init_co2: timenee = ',timenee

    ! average/interpolate to footprint time
    if ( ndgrid.eq.nd_nee ) then
      ! no averaging/interpolation needed
      fluxes%flxnee_nest(:,:,(nd-1)*int(statres)*ndgrid+1:nd*int(statres)*ndgrid) = reg_flx(:,:,:)
    else
      ! do averaging/interpolation
      allocate( datain(nxregrid,int(statres)*nd_nee) )
      allocate( dataout(nxregrid,int(statres)*ndgrid) )
      allocate( timefp(int(statres)*ndgrid) )
      timefp(:) = fluxes%timefp((nd-1)*int(statres)*ndgrid+1:nd*int(statres)*ndgrid)
      if ( ndgrid.lt.nd_nee ) then
        ! average flux
        do jy = 1, nyregrid
          datain = reg_flx(:,jy,:)
          call average(nxregrid, int(statres)*nd_nee, timenee, datain, int(statres)*ndgrid, timefp, dataout)
          fluxes%flxnee_nest(:,jy,(nd-1)*int(statres)*ndgrid+1:nd*int(statres)*ndgrid) = dataout(:,:)
        end do
      else if ( ndgrid.gt.nd_nee ) then
        ! interpolate flux
        do jy = 1, nyregrid
          datain = reg_flx(:,jy,:)
          call interp(nxregrid, int(statres)*nd_nee, timenee, datain, int(statres)*ndgrid, timefp, dataout)
          fluxes%flxnee_nest(:,jy,(nd-1)*int(statres)*ndgrid+1:nd*int(statres)*ndgrid) = dataout(:,:)
        end do
      endif
      deallocate( datain )
      deallocate( dataout )
      deallocate( timefp )
    endif

    ! average hourly NEE over statres dimension
    reg_nee(:,:,:) = 0.
    do i = 1, int(statres)
      reg_nee(:,:,:) = reg_nee(:,:,:) + reg_flx(:,:,(i-1)*nd_nee+1:i*nd_nee)
    end do
    reg_nee(:,:,:) = reg_nee(:,:,:)/statres

    ! calculate area-weighted mean NEE for each box
    do n = 1, nd_nee
      do jy = 1, nyregrid
        area = grid_area(reg_lat(jy)+0.5*rdy, rdy, rdx)
        do ix = 1, nxregrid
          if ( nbox_xy(ix,jy).gt.0 ) then
            nee_box(nbox_xy(ix,jy),nd,n) = nee_box(nbox_xy(ix,jy),nd,n) + reg_nee(ix,jy,n) * area
          endif
        end do
      end do
      nee_box(:,nd,n) = nee_box(:,nd,n)/area_box
    end do

    ! calculate prior NEE uncertainty for each box
    do nb = 1, nbox
      cnt(:) = 0
      do n = 1, nd_nee
        hh = (n-1)*24/nd_nee + int(hloc(nb))
        if ( hh.lt.0 ) then
          hh = hh + 24
        else if ( hh.ge.24 ) then
          hh = hh - 24
        endif
        do nt = 1, ndt
          if ( (hh.ge.(nt-1)*24/ndt).and.(hh.lt.nt*24/ndt) ) then
            cnt(nt) = cnt(nt) + 1
            err_box(nb,nd,nt) = err_box(nb,nd,nt) + nee_box(nb,nd,n)*config%flxerr
          endif
        end do
      end do
      do nt = 1, ndt
        err_box(nb,nd,nt) = err_box(nb,nd,nt)/cnt(nt)
      end do
    end do

    statime(nd) = jd
    nd = nd + 1
    jd = jd + real(statres,kind=8)

  end do

  ! verbose output for testing
  ! --------------------------

  if ( config%verbose ) then
    ! NEE for region interpolated to footprint times
    filename = trim(files%path_output)//'flxnee_nest.txt'
    write(rowfmt,'(A,I6,A)') '(',nxregrid,'(E11.4,1X))'
    open(100,file=trim(filename),status='replace',action='write',iostat=ierr)
    do nd = 1, nday*ndgrid
      do n = 1, nyregrid
        write(100,fmt=rowfmt) fluxes%flxnee_nest(:,n,nd)
      end do
    end do
    close(100)
    ! NEE for region before interpolation (last statres timestep only)
    filename = trim(files%path_output)//'reg_flx.txt'
    write(rowfmt,'(A,I6,A)') '(',nxregrid,'(E11.4,1X))'
    open(100,file=trim(filename),status='replace',action='write',iostat=ierr)
    do nd = 1, int(statres)*nd_nee
      do n = 1, nyregrid
        write(100,fmt=rowfmt) reg_flx(:,n,nd)
      end do
    end do
    close(100)  
    ! NEE for boxes
    filename = trim(files%path_output)//'nee_box.txt'
    write(rowfmt,'(A,I6,A)') '(',nbox,'(E11.4,1X))'
    open(100,file=trim(filename),status='replace',action='write',iostat=ierr)
    do nd = 1, ntstate
      do n = 1, nd_nee
        write(100,fmt=rowfmt) nee_box(:,nd,n)
      end do
    end do
    close(100)
    ! NEE uncertainty for boxes  
    write(rowfmt,'(A,I6,A)') '(',nbox,'(E11.4,1X))'
    open(100,file=trim(files%path_output)//'err_box.txt',status='replace',action='write',iostat=ierr)
    do nd = 1, ntstate
      do n = 1, ndt
        write(100,fmt=rowfmt) err_box(:,nd,n)
      end do
    end do
    close(100)
  endif

  ! prior state vector
  ! ------------------

  ! number of state variables
  npvar = ndt*ntstate*nbox
  ndvar = ndt*nbox
  if ( config%opt_cini ) then
    nvar = npvar + ntcini*ncini
  else
    nvar = npvar
  endif

  write(logid,*) 'Number of flux state variables: ',npvar
  write(logid,*) 'Number of state variables per flux time step: ',ndvar
  write(logid,*) 'Total number of state variables: ',nvar

  call alloc_states(states) 
  allocate( xerr(ndvar) )

  ! assign prior state vector
  states%px0(:) = 0.
  if ( config%opt_cini ) states%px0(npvar+1:nvar) = 1.

  ! assign time stamps to flux time steps
  states%xtime(:) = statime
  print*, 'init_co2: states%xtime = ',states%xtime

  ! assign time stamps to mixing ratio scalar time steps
  do n = 1, ntcini
    states%cinitime(n) = juldatei + dble((n-1)*cinires)
  end do
  print*, 'init_co2: cinitime = ',states%cinitime

  ! assign best guess of state vector
  if ( (config%prior_bg.eq.1).and.(config%method.eq.'congrad') ) then
    ! from previous inversion
    call read_bg(files, config, states)
  else
    ! from prior
    states%px(:) = states%px0(:)
  endif

  ! prior state vector uncertainty
  do n = 1,ndt
    xerr((n-1)*nbox+1:n*nbox) = abs(sum(err_box(:,:,n),dim=2)/ntstate)
  end do

  if ( config%verbose ) then
    ! state vector uncertainty   
    open(100,file=trim(files%path_output)//'xerr.txt',status='replace',action='write',iostat=ierr)
    write(rowfmt,'(A,I6,A)') '(',nbox,'(E11.4,1X))'
    do n = 1, ndt
      write(100,fmt=rowfmt) xerr((n-1)*nbox+1:n*nbox)
    end do
    close(100)
  endif

  ! correlation and error covariance matrices
  ! -----------------------------------------

  corr(:,:) = 0.
  if ( config%spa_corr ) then
    write(logid,*) 'Using spatial correlation in B'
    ! check if file already exists (in case of rerun)
    inquire(file=trim(files%path_output)//'correl.txt',exist=lexist)
    if ( lexist ) then
      write(logid,*) 'Reading spatial correlation from correl.txt'
      open(100,file=trim(files%path_output)//'correl.txt',action='read',status='old',iostat=ierr)
      write(rowfmt,'(A,I6,A)') '(',nbox,'(E11.4,1X))'
      do nb = 1, nbox
        read(100,rowfmt) corr(nb,:)
      end do
      close(100)
    else
      call correl(files, config, corr)
    endif
  else
    do n = 1, nbox
      corr(n,n) = 1.
    end do
  endif
  call error_cov(config, files, states, covar, corr, xerr)

  ! observations
  ! ------------

  nobs = 0
  call read_obs(config, files)
  call assign_obs(config, files, obs)

  ! initial concentrations
  ! ----------------------

  ! initial concentrations from termination of trajectories
  ! only do for a fresh run
  if ( config%restart.eq.0 ) then
    call init_cini(files, config, obs)
!    call init_cams(files, config, obs)
!    call init_fpctm_hour(files, config, obs)
  endif
    
  ! add random perturbation for MC
  ! ------------------------------
   
  if ( config%run_mode.eq.2 ) then
    write(logid,*) 'Adding random perturbation to state vector and observations...'
    ! perturb state vector
    call perturb_state(config, states, covar)
    ! perturb observations 
    call perturb_obs(config, obs)
  endif

  ! save datatype obs for restart
  ! -----------------------------

  if ( config%restart.eq.1 ) then
    ! read datatype obs from file
    write(logid,*) 'WARNING init_co2: reading cini from previous run'
    open(100,file=trim(files%path_output)//'obsdatatype.txt',status='old',action='read',iostat=ierr)
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR init_co2: cannot open file obsdatatype.txt'
      stop
    endif
    allocate( recs(nobs) )
    allocate( jdate(nobs) )
    write(rowfmt,'(A,I6,A)') '(A3,1X,F14.6,1X,F10.4,1X,',ncini,'(F10.4,1X))'
    read(100,*) dump
    do n = 1, nobs
      read(100,rowfmt,iostat=ierr) recs(n), jdate(n), conc, obs%cini(n,:)
      if ( ierr.ne.0 ) then
        write(logid,*) 'ERROR init_co2: reading obsdatatype.txt'
        exit
      endif
    end do
    ! check consistency
    if ( recs(1).ne.obs%recs(1).or.recs(nobs).ne.obs%recs(nobs).or.jdate(1).ne.obs%jdate(1).or.jdate(nobs).ne.obs%jdate(nobs) ) then
      write(logid,*) 'ERROR init_co2: data in obsdatatype.txt inconsistent with run'
      stop
    endif  
    deallocate( recs )
    deallocate( jdate )
  else
    ! save datatype obs
    call save_for_restart(files, obs)  
  endif

end subroutine init_co2


