!---------------------------------------------------------------------------------------
! FLEXINVERT: init_fpctm_hour
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> init_fpctm_hour
!!
!! Purpose:    Calculates the initial concentration for each observation using the
!!             4D initial concentration fields and the flexpart grid_initial files.
!!             Note: adapted from init_cini for FP-CTM hourly output
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             config  -  configuration settings data structure
!!             obs     -  observation data structure 
!!
!!    Externals
!!             caldate
!!             read_init              
!!             get_initfpctm_hour
!!
!---------------------------------------------------------------------------------------

subroutine init_fpctm_hour(files, config, obs)

  use mod_flexpart
  use mod_settings
  use mod_strings
  use mod_var
  use mod_dates
  use mod_obs
  use mod_tools
  use netcdf
  use mod_save,   only : check

  implicit none

  type (files_t),        intent (in)     :: files
  type (config_t),       intent (in)     :: config
  type (obs_t),          intent (in out) :: obs

  character(max_path_len)                :: path_flexrec, pathname, filename
  character(max_name_len)                :: varname
  character(len=8)                       :: adate, ayear, areldate, areltime, amonth, atime
  logical                                :: lexist
  integer                                :: jjjjmm, jjjjmmdd, hhmiss, lasttime, currenttime
  integer                                :: ix, jy, kz, i, n, ncid, tid, it
  integer                                :: month, prevmonth
  real(kind=8)                           :: jdi, hh
  real(kind=8), dimension(nobs)          :: jdates
  real(kind=8), dimension(:), allocatable:: time
  real, dimension(nobs,ncini)            :: cini
  integer, dimension(nobs)               :: ind
  real, dimension(nxgrid,nygrid,nzgrid)  :: gridini, gridini_tmp
  real, dimension(:,:,:,:), allocatable  :: concini

  real                                   :: bndx, bndy, delx, dely
  integer                                :: numx, numy, xshift
  integer                                :: numpoint, release_nr, nr_init
  integer                                :: ibdate, ibtime
  real(kind=8), dimension(maxpoint,2)    :: releases


  ! loop over observations
  ! ----------------------

  cini(:,:) = 0.
  lasttime = 0
  prevmonth = 0

  ! sort obs chronologically for efficiency
  jdates = obs%jdate(:)
  call sort(nobs,jdates,ind)

  do i = 1, nobs

    ! reset values
    gridini(:,:,:) = 0.
    nr_init = 0

    ! check month of observation
    call caldate(jdates(i), jjjjmmdd, hhmiss)
    write(adate,'(I6.6)') jjjjmmdd/100
    month = jjjjmmdd/100
    path_flexrec = trim(files%path_flexpart)//trim(obs%recs(ind(i)))//'/'//trim(adate)//'/'
    print*, 'init_fpctm: pathflexrec = ',path_flexrec

    if ( config%average_fp ) then

      ! match and, if needed, average grid_initial files to observation timestamp

      ! get header and release information related to current observation location 
      ! (if not already read and same for previous observation)
      filename = trim(path_flexrec)//'header'
      if ( i.eq.1 .or. month.ne.prevmonth ) then
        write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
        call read_header(filename, numpoint, ibdate, ibtime, releases, &
                           numx, numy, bndx, bndy, delx, dely, xshift)
      else
        if(trim(obs%recs(ind(i))).ne.trim(obs%recs(ind(i-1))))then
          write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
          call read_header(filename, numpoint, ibdate, ibtime, releases, &
                             numx, numy, bndx, bndy, delx, dely, xshift)
        endif
      endif
      ! find the first release number corresponding to the current observation
      release_nr=1
      do while ( int(releases(release_nr,1)*1e4,kind=8).lt.int(jdates(i)*1e4,kind=8) .and. release_nr.lt.numpoint )
        release_nr = release_nr+1
      end do
      ! define first filename
      call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
      ! round seconds
      hhmiss = (hhmiss/100)*100
      write(areldate,'(I8.8)') jjjjmmdd
      write(areltime,'(I6.6)') hhmiss
      filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
      print*, 'init_fpctm: filename = ',filename
      ! read first flexpart grid_init files
      inquire ( file=trim(filename), exist=lexist )
      if ( lexist ) then
        call read_init(filename, gridini_tmp)
        gridini = gridini + gridini_tmp
        nr_init = 1
        ! current release_nr was already read, start with next
        release_nr = release_nr + 1
        do while ( int(releases(release_nr,2)*1e4,kind=8).le.int((jdates(i)+obs%avetime(ind(i)))*1e4,kind=8) .and. release_nr.lt.numpoint )
          call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
          write(adate,'(I6.6)') jjjjmmdd/100
          write(areldate,'(I8.8)') jjjjmmdd
          hhmiss=int(hhmiss/100)*100
          write(areltime,'(I6.6)') hhmiss
          path_flexrec = trim(files%path_flexpart)//trim(obs%recs(ind(i)))//'/'//trim(adate)//'/'
          filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
          ! sum the grid with previous initial grids
          call read_init(filename, gridini_tmp)
          gridini = gridini + gridini_tmp
          release_nr = release_nr+1
          nr_init = nr_init + 1
        end do ! finished summing grid_initial
        ! convert from ppt to fraction of one
        gridini = gridini*1.e-12
        gridini = gridini/real(nr_init)
      else
        ! filename not found 
        write(logid,*) 'WARNING: cannot find '//trim(filename)
        go to 10
      endif

    else

      ! timestamps of grid_init files match observations

      ! define file name
      write(areldate,'(I8.8)') jjjjmmdd
      write(areltime,'(I6.6)') hhmiss
      filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
      print*, 'init_fpctm: filename = ',filename
      ! read flexpart grid_init file
      inquire(file=trim(filename),exist=lexist)
      if(lexist) then
        call read_init(filename, gridini)
        ! convert from ppt to fraction of one
        gridini = gridini*1.e-12
      else
        write(logid,*) 'WARNING: cannot find ',filename
        go to 10
      endif

    endif 

    ! calculate termination time of trajectory
    jdi = jdates(i) - trajdays
    call caldate(jdi, jjjjmmdd, hhmiss)
    jjjjmm = jjjjmmdd/100
    ! round time to nearest 3-hours
    hh = (jdi - floor(jdi))*24d0
    hh = floor(hh/3d0)*3d0
   
    ! read concentration field only if not already in memory
    currenttime = jjjjmm*100
    if ( currenttime.ne.lasttime ) then
      ! read new concentration field
      write(adate,'(I8.8)') jjjjmmdd
      write(ayear,'(I4.4)') jjjjmm/100
      write(amonth,'(I6.6)') jjjjmm
      write(atime,'(I2.2)') int(hh)
      pathname = str_replace(files%path_initconc,'YYYY', trim(ayear))
      filename = str_replace(files%file_initconc, 'YYYYMM', trim(amonth))
      filename = trim(pathname)//trim(filename)
      inquire(file=trim(filename),exist=lexist)
      print*, filename
      if(lexist) then
        ! read time dimension from file (nztime is global)
        call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
        call check( nf90_inq_dimid(ncid,'Date',tid) )
        call check( nf90_inquire_dimension(ncid,tid,len=nztime) )
        if( allocated(time) ) deallocate(time)
        allocate( time(nztime) )
        call check( nf90_inq_varid(ncid,'Date',tid) )
        call check( nf90_get_var(ncid,tid,time) )
        call check( nf90_close(ncid) )
        ! read conc and interpolate to flexpart grid
        if( allocated(concini) ) deallocate(concini)
        allocate( concini(nxgrid,nygrid,nzgrid,nztime) )
        varname = files%varname_init
        call get_initfpctm_hour(filename, varname, concini)
      else
        write(logid,*) 'WARNING: cannot find ',filename
        go to 10
      endif
    endif

    ! calculate contribution of initial concentration to receptor
!    it = minloc(abs(time-jdates(i)),dim=1,mask=abs(time-jdates(i)).eq.minval(abs(time-jdates(i))))
    it = minloc(abs(time-jdi),dim=1,mask=abs(time-jdi).eq.minval(abs(time-jdi)))
    do kz = 1, nzgrid
      do jy = 1, nygrid
        n = 0
        do while(.true.)
          n = n + 1
          if ( gbl_lat(jy).lt.cini_lat(n) ) exit
        end do
        do ix = 1, nxgrid
          cini(i,n) = cini(i,n) + gridini(ix,jy,kz)*concini(ix,jy,kz,it)
        end do
      end do
    end do

10  continue

    lasttime = currenttime

  end do

  ! reorder cini for observations
  do i = 1, nobs
    obs%cini(ind(i),:) = cini(i,:)
  end do

end subroutine init_fpctm_hour


